
/* i/o functions */

#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdarg.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "io.h"
#include "config.h"

static struct addrinfo * gethostinfo(char const *host, int port);

int SockOpen(const char *host, int clientPort)
{
    int sock = -1;
    struct addrinfo *hostres, *res;

    hostres = gethostinfo(host, clientPort);
    for (res = hostres; res; res = res->ai_next)
    {
      sock = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
      if (sock < 0)
        continue;
      if (connect(sock, res->ai_addr, res->ai_addrlen) == 0)
        break;
      close(sock);
      return -errno;
    }
    freeaddrinfo(hostres);
    if (res == NULL)
      return -1;

    return sock;
}

int SockPrintf(int sock, char *format,...)
{
    va_list ap;
    char buf[8192];

    va_start(ap, format);
#ifdef HAVE_VSNPRINTF
    vsnprintf(buf, sizeof(buf), format, ap);
#else
    vsprintf(buf, format, ap);
#endif
    va_end(ap);

    return SockWrite(sock, buf, strlen(buf));

}

int SockWrite(int sock, char *buf, int len)
{
    int n, wrlen = 0;

    while (len)
    {
	n = write(sock, buf, len);
	if (n <= 0)
	    return -1;
	len -= n;
	wrlen += n;
	buf += n;
    }
    return wrlen;
}

int SockRead(int sock, char *buf, int len)
{
    char *newline = NULL, *bp = buf;
    int n;

    if (--len < 1)
	return (-1);
    do
    {
	/* 
	 * The reason for these gymnastics is that we want two things:
	 * (1) to read \n-terminated lines,
	 * (2) to return the true length of data read, even if the
	 *     data coming in has embedded NULS.
	 */
	if ((n = recv(sock, bp, len, MSG_PEEK)) <= 0)
	{
	    if (errno == EINTR)
		continue;
	    else
		return (-1);
	}
	if ((newline = memchr(bp, '\n', n)) != NULL)
	    n = newline - bp + 1;

	if ((n = read(sock, bp, n)) == -1)
	    return (-1);

	bp += n;
	len -= n;
    }
    while
	(!newline && len);
    *bp = '\0';

    return bp - buf;
}

void AddDebug(char *cbuf)
{
#ifdef SDEBUG
    printf("%s", cbuf);
    fflush(stdout);
#endif
}

/* Stolen from FreeBSD's whois client, modified for jupe by W. Campbell */
static struct addrinfo * gethostinfo(char const *host, int port)
{
        struct addrinfo hints, *res;
        int error;
        char portbuf[6];

        memset(&hints, 0, sizeof(hints));
        hints.ai_flags = 0;
        hints.ai_family = AF_UNSPEC;
        hints.ai_socktype = SOCK_STREAM;
        snprintf(portbuf, 6, "%d", port);
        error = getaddrinfo(host, portbuf, &hints, &res);
        if (error)
                return (NULL);
        return (res);
}
